@@ -23,11 +23,17 @@
 #' @param ... Additional attributes.
 #' @param width The maximum column width.
 #' @param min_width The minimum allowed column width, `width` if omitted.
+#' @param type_sum `r lifecycle::badge("experimental")`
+#'
+#'   Override the type summary displayed at the top of the data.
+#'   This argument, if given, takes precedence over the type summary provided by
+#'   [type_sum()].
 #' @param class The name of the subclass.
 #' @param subclass Deprecated, pass the `class` argument instead.
 #' @name new_pillar_shaft
 #' @export
-new_pillar_shaft <- function(x, ..., width = NULL, min_width = width, class = NULL, subclass = NULL) {
+new_pillar_shaft <- function(x, ..., width = NULL, min_width = width,
+                             type_sum = NULL, class = NULL, subclass = NULL) {
   if (!is.null(subclass)) {
     deprecate_soft("1.4.0", "pillar::new_pillar_shaft(subclass = )", "new_pillar_shaft(class = )")
     class <- subclass
@@ -40,6 +46,7 @@ new_pillar_shaft <- function(x, ..., width = NULL, min_width = width, class = NU
   ret <- structure(
     x,
     ...,
+    type = if (!is.null(type_sum)) new_pillar_type_obj(type_sum),
     class = c(class, "pillar_shaft")
   )
   ret <- set_width(ret, width)
@@ -54,7 +61,7 @@ new_pillar_shaft <- function(x, ..., width = NULL, min_width = width, class = NU
 #' for your data type to display it in a tibble.
 #'
 #' @param x A vector to format
-#' @inheritParams ellipsis::dots_used
+#' @inheritParams rlang::args_dots_used
 #' @export
 #' @examples
 #' pillar_shaft(1:3)
@@ -116,14 +123,17 @@ pillar_shaft.logical <- function(x, ...) {
 #' @param sigfig
 #'   Deprecated, use [num()] or [set_num_opts()] on the data instead.
 pillar_shaft.numeric <- function(x, ..., sigfig = NULL) {
-  pillar_attr <- attr(x, "pillar")
+  pillar_attr <- attr(x, "pillar", exact = TRUE)
 
-  if (is.null(pillar_attr) && !is.null(attr(x, "class"))) {
+  if (is.null(pillar_attr) && !is.null(attr(x, "class", exact = TRUE))) {
     ret <- format(x)
     return(new_pillar_shaft_simple(ret, width = get_max_extent(ret), align = "left"))
   }
 
-  data <- unclass(x)
+  pillar_shaft_number_attr(unclass(x), pillar_attr, sigfig)
+}
+
+pillar_shaft_number_attr <- function(data, pillar_attr, sigfig = NULL) {
   scale <- pillar_attr$scale
   if (!is.null(scale)) {
     data <- data * scale
@@ -134,30 +144,30 @@ pillar_shaft.numeric <- function(x, ..., sigfig = NULL) {
     sigfig %||% pillar_attr$sigfig,
     pillar_attr$digits,
     pillar_attr$notation,
-    pillar_attr$fixed_exponent
+    pillar_attr$fixed_exponent,
+    pillar_attr$extra_sigfig
   )
 }
 
-pillar_shaft_number <- function(x, sigfig, digits, notation, fixed_exponent) {
+pillar_shaft_number <- function(x, sigfig, digits, notation, fixed_exponent, extra_sigfig) {
   if (!is.null(digits)) {
     if (!is.numeric(digits) || length(digits) != 1) {
       abort("`digits` must be a number.")
     }
   }
   if (is.null(sigfig)) {
-    sigfig <- getOption("pillar.sigfig", 3)
-    if (!is.numeric(sigfig) || length(sigfig) != 1 || sigfig < 1L) {
-      inform("Option pillar.sigfig must be a positive number greater or equal 1. Resetting to 1.")
-      sigfig <- 1L
-      options(pillar.sigfig = sigfig)
-    }
+    sigfig <- get_pillar_option_sigfig()
+  }
+
+  if (isTRUE(extra_sigfig)) {
+    sigfig <- sigfig + compute_extra_sigfig(x)
   }
 
   if (is.null(notation) || notation == "fit") {
     dec <- split_decimal(x, sigfig = sigfig, digits = digits)
     sci <- split_decimal(x, sigfig = sigfig, digits = digits, sci_mod = 1, fixed_exponent = fixed_exponent)
 
-    max_dec_width <- getOption("pillar.max_dec_width", 13)
+    max_dec_width <- get_pillar_option_max_dec_width()
     dec_width <- get_width(dec)
     "!!!!!!DEBUG `v(dec_width)`"
 
@@ -197,35 +207,52 @@ pillar_shaft_number <- function(x, sigfig, digits, notation, fixed_exponent) {
   ret$dec <- dec
   ret$sci <- sci
 
+  if (!is.null(ret$sci$unit) && ret$sci$unit != 0) {
+    type_sum <- I(paste0(
+      # Turn off subtle styling for units
+      sub(" ", "", style_subtle(" ")),
+      "[",
+      format_exp(ret$sci$unit, (notation == "si")),
+      "]"
+    ))
+  } else {
+    type_sum <- NULL
+  }
+
   new_pillar_shaft(
     ret,
     width = get_width(ret$dec %||% ret$sci),
     min_width = min(get_min_widths(ret)),
+    type_sum = type_sum,
     class = "pillar_shaft_decimal"
   )
 }
 
-# registered in .onLoad()
+#' @export
 pillar_shaft.integer64 <- function(x, ..., sigfig = NULL) {
-  pillar_shaft_number(x, sigfig, digits = NULL, notation = NULL, fixed_exponent = NULL)
+  if (class(x)[[1]] != "integer64") {
+    return(NextMethod())
+  }
+
+  pillar_shaft_number(x, sigfig, digits = NULL, notation = NULL, fixed_exponent = NULL, extra_sigfig = NULL)
 }
 
-# registered in .onLoad()
+#' @export
 pillar_shaft.Surv <- function(x, ...) {
   new_pillar_shaft_simple(format(x), align = "right")
 }
 
-# registered in .onLoad()
+#' @export
 pillar_shaft.Surv2 <- function(x, ...) {
   new_pillar_shaft_simple(format(x), align = "right")
 }
 
-# registered in .onLoad()
+#' @export
 type_sum.Surv <- function(x) {
   "Surv"
 }
 
-# registered in .onLoad()
+#' @export
 type_sum.Surv2 <- function(x) {
   "Surv2"
 }
@@ -262,7 +289,7 @@ pillar_shaft.POSIXt <- function(x, ...) {
 #' @param min_width
 #'   Deprecated, use [char()] or [set_char_opts()] on the data instead.
 pillar_shaft.character <- function(x, ..., min_width = NULL) {
-  pillar_attr <- attr(x, "pillar")
+  pillar_attr <- attr(x, "pillar", exact = TRUE)
 
   min_chars <- min_width %||% pillar_attr$min_chars
 
@@ -272,8 +299,10 @@ pillar_shaft.character <- function(x, ..., min_width = NULL) {
   # Add subtle quotes if necessary
   needs_quotes <- which(is_ambiguous_string(x))
   if (has_length(needs_quotes)) {
-    out[needs_quotes] <- gsub('"', '\\"', x[needs_quotes], fixed = TRUE)
-    out[!is.na(x)] <- paste0(style_subtle('"'), out[!is.na(x)], style_subtle('"'))
+    quoted <- gsub('"', '\\"', x[needs_quotes], fixed = TRUE)
+    out[needs_quotes] <- quoted
+    valid <- paste0(style_subtle('"'), out[!is.na(x)], style_subtle('"'))
+    out[!is.na(x)] <- valid
     na_indent <- 1
   } else {
     na_indent <- 0
@@ -281,29 +310,26 @@ pillar_shaft.character <- function(x, ..., min_width = NULL) {
 
   # determine width based on width of characters in the vector
   if (is.null(min_chars)) {
-    min_chars <- getOption("pillar.min_chars", 3L)
-    if (!is.numeric(min_chars) || length(min_chars) != 1 || min_chars < 3L) {
-      inform("Option pillar.min_chars must be a nonnegative number greater or equal 3. Resetting to 3.")
-      min_chars <- 3L
-      options(pillar.min_chars = min_chars)
-    }
+    min_chars <- get_pillar_option_min_chars()
   }
 
-  pillar_shaft(new_vertical(out), ..., min_width = min_chars, na_indent = na_indent, shorten = pillar_attr$shorten)
+  pillar_shaft(as_glue(out), ..., min_width = min_chars, na_indent = na_indent, shorten = pillar_attr$shorten)
 }
 
 #' @export
 #' @inheritParams new_pillar_shaft_simple
 #' @rdname pillar_shaft
-pillar_shaft.pillar_vertical <- function(x, ..., min_width = NULL, na_indent = 0L, shorten = NULL) {
+pillar_shaft.glue <- function(x, ..., min_width = NULL, na_indent = 0L, shorten = NULL) {
   min_width <- max(min_width, 3L)
   width <- get_max_extent(x)
+  na <- pillar_na(use_brackets_if_no_color = TRUE)
+  force(na_indent)
 
   new_pillar_shaft_simple(
     x,
     width = width, align = "left", min_width = min(width, min_width),
-    na = pillar_na(use_brackets_if_no_color = TRUE),
-    na_indent = na_indent,
+    na = na,
+    na_indent = if (width > get_extent(na)) na_indent else 0,
     shorten = shorten
   )
 }
@@ -311,11 +337,25 @@ pillar_shaft.pillar_vertical <- function(x, ..., min_width = NULL, na_indent = 0
 #' @export
 #' @rdname pillar_shaft
 pillar_shaft.list <- function(x, ...) {
-  out <- paste0("<", map_chr(x, obj_sum), ">")
+  summary <- map(x, obj_sum)
 
-  width <- get_max_extent(out)
+  formatted <- paste0("<", unlist(summary), ">")
 
-  new_pillar_shaft_simple(style_list(out), width = width, align = "left", min_width = min(width, 3L))
+  short <- map(summary, attr, "short")
+  short_idx <- !map_lgl(short, is.null)
+  short_formatted <- formatted
+  short_formatted[short_idx] <- paste0("<", unlist(short[short_idx]), ">")
+
+  width <- get_max_extent(formatted)
+  min_width <- get_max_extent(short_formatted)
+
+  new_pillar_shaft_simple(
+    style_list(formatted),
+    width = width,
+    align = "left",
+    min_width = min_width,
+    short_formatted = short_formatted
+  )
 }
 
 #' @export
@@ -336,5 +376,5 @@ pillar_shaft.default <- function(x, ...) {
   #' @details
   #' The default method will currently format via [format()],
   #' but you should not rely on this behavior.
-  pillar_shaft(new_vertical(format(x)), ...)
+  pillar_shaft(as_glue(format(x)), ...)
 }
