@@ -33,6 +33,10 @@ tbl_format_header <- function(x, setup, ...) {
 #' @export
 tbl_format_header.tbl <- function(x, setup, ...) {
   named_header <- setup$tbl_sum
+  focus <- attr(x, "pillar_focus")
+  if (!is.null(focus)) {
+    named_header <- c(named_header, "Focus columns" = collapse(tick_if_needed(focus)))
+  }
 
   if (all(names2(named_header) == "")) {
     header <- named_header
@@ -51,7 +55,7 @@ tbl_format_header.tbl <- function(x, setup, ...) {
 
 #' @export
 tbl_format_header.pillar_tbl_format_setup <- function(x, ...) {
-  new_vertical(c(
+  as_glue(c(
     cli::style_bold("<tbl_format_header(setup)>"),
     tbl_format_header(x$x, setup = x)
   ))
